/*****************************************************************************
 * main.c - Live! Drive IR Remote control decoder
 *****************************************************************************
 * Copyright (C) 2001 Arnaud de Bossoreille de Ribou
 *
 * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/


/*****************************************************************************
 * Description :
 *
 * this program decodes the keys pressed on the sblive IR remote control.
 *****************************************************************************/


#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>

#include <xmms/xmmsctrl.h>

#include "common.h"
#include "douze.h"
#include "douze_parser.h"
#include "livedrive_decoder.h"

#define xmms_session 0


/*****************************************************************************
 * netlist size (1 packets of 13 bytes)
 *****************************************************************************/
#define DD_PACKET_SIZE      13
#define DD_LIFO_SIZE        1


/*****************************************************************************
 * main_thread_t
 *****************************************************************************
 * This structure is used to store global data of the program.
 *****************************************************************************/
typedef struct main_thread_s
{
  douze_demux_t         demux;
  douze_parser_t        parser;
  livedrive_decoder_t   decoder;
  boolean_t             b_stop;
  int                   i_fd;
  byte_t                buffer[DD_PACKET_SIZE * DD_LIFO_SIZE];
  douze_packet_t        p_packets[DD_LIFO_SIZE];
  douze_packet_t *      lifo[DD_LIFO_SIZE];
  unsigned int          ui_next;

} main_thread_t;


/*****************************************************************************
 * main structure.
 *****************************************************************************/
static main_thread_t * s_p_main = NULL;


/*****************************************************************************
 * parser callback functions
 *****************************************************************************/
void HandleKey(void * p_data, u32 ui_key_id)
{
  static int xmms_volume;
  static gboolean xmms_muted = FALSE;
  
  creative_key_desc_t * p_key_desc =
                        creative_FindKeyDesc(ui_key_id);
  creative_dev_desc_t * p_dev_desc =
                        creative_FindDevDesc(p_key_desc->ui_dev_id);

  fprintf(stdout, "Creative key pressed (id = 0x%08x)\n", ui_key_id);
  fprintf(stdout, "  - device \"%s\"\n", p_dev_desc->psz_desc);
  fprintf(stdout, "  - key    \"%s\"\n", p_key_desc->psz_desc);
  fprintf(stdout, "\n");
  
  /*** XMMS Control ***/
  
  
  
  /* Searching */
  if (!strcmp(p_key_desc->psz_desc, "Right")) {
  	int pos;
  	pos = xmms_remote_get_output_time(xmms_session);
  	pos += 5000;
  	printf ("%s: output time: %d\n", __FUNCTION__, pos);
  	xmms_remote_jump_to_time(xmms_session, pos);
  }
  if (!strcmp(p_key_desc->psz_desc, "Left")) {
  	int pos;
  	pos = xmms_remote_get_output_time(xmms_session);
  	pos -= 5000;
  	xmms_remote_jump_to_time(xmms_session, pos);
  }
  

  /* Playlist */
  if (!strcmp(p_key_desc->psz_desc, "Next")) xmms_remote_playlist_next(xmms_session);
  if (!strcmp(p_key_desc->psz_desc, "Previous")) xmms_remote_playlist_prev(xmms_session);
  if (!strcmp(p_key_desc->psz_desc, "Step")) xmms_remote_toggle_shuffle(xmms_session);;
  
  if (!strcmp(p_key_desc->psz_desc, "Up")) {
  	int pos;
  	pos = xmms_remote_get_playlist_pos(xmms_session);
  	pos--;
  	xmms_remote_set_playlist_pos(xmms_session, pos);
  }
  if (!strcmp(p_key_desc->psz_desc, "Down")) {
  	int pos;
  	pos = xmms_remote_get_playlist_pos(xmms_session);
  	pos++;
  	xmms_remote_set_playlist_pos(xmms_session, pos);
  }
  
  
  /* Volume */
  if (!strcmp(p_key_desc->psz_desc, "Volume +")) {
  	if (!xmms_muted) xmms_volume = xmms_remote_get_main_volume (xmms_session);
  	if (xmms_muted) xmms_muted = FALSE;
  	xmms_volume += 2;
  	xmms_remote_set_main_volume (xmms_session, xmms_volume);
  }
  if (!strcmp(p_key_desc->psz_desc, "Volume -")) {
  	if (!xmms_muted) xmms_volume = xmms_remote_get_main_volume (xmms_session);
  	if (xmms_muted) xmms_muted = FALSE;
  	xmms_volume -= 2;
  	xmms_remote_set_main_volume (xmms_session, xmms_volume);
  }
  if (!strcmp(p_key_desc->psz_desc, "Mute")) {
  	if (xmms_muted) {
  		xmms_muted = FALSE;
  		xmms_remote_set_main_volume(xmms_session, xmms_volume);
  	} else {
  		xmms_muted = TRUE;
  		xmms_volume = xmms_remote_get_main_volume (xmms_session);
  		xmms_remote_set_main_volume (xmms_session, 0);
  	}
  }
  
  
  /* Control */
  if (!strcmp(p_key_desc->psz_desc, "Play/Pause")) { 
  	if (xmms_remote_is_playing(xmms_session))
  		xmms_remote_pause(xmms_session);
  	else
  		xmms_remote_play(xmms_session);
  }
  if (!strcmp(p_key_desc->psz_desc, "Stop")) xmms_remote_stop(xmms_session);

}


/*****************************************************************************
 * GetPacket
 *****************************************************************************
 * This function gats a free packet in the netlist.
 *****************************************************************************/
douze_packet_t * GetPacket(main_thread_t * p_main)
{
  if(p_main->ui_next < DD_LIFO_SIZE)
  {
    douze_packet_t * p_packet = p_main->lifo[p_main->ui_next++];
    return p_packet;
  }
  else
    return NULL;
}


/*****************************************************************************
 * ReleasePacket
 *****************************************************************************
 * This function queues the released packet into the netlist.
 *****************************************************************************/
void ReleasePacket(main_thread_t * p_main, douze_packet_t * p_packet)
{
  p_main->lifo[--p_main->ui_next] = p_packet;
}


/*****************************************************************************
 * SignalHandler
 *****************************************************************************/
void SignalHandler(int i_signal)
{
  switch(i_signal)
  {
  case SIGINT:
    fprintf(stderr, LIVEDRIVE_DECODER "SIGINT signal received\n");
    if(s_p_main)
      s_p_main->b_stop = 1;
    break;
  default:
    fprintf(stderr, LIVEDRIVE_DECODER
                    "unhandled signal received (%d)\n", i_signal);
    break;
  }
}


/*****************************************************************************
 * InstallSignalHandlers
 *****************************************************************************
 * This function installs a handler for system signals.
 *****************************************************************************/
int InstallSignalHandlers()
{
  sigset_t sSigSet;
  struct sigaction sSigAction;

  sigemptyset(&sSigSet);
  sSigAction.sa_mask = sSigSet;
  sSigAction.sa_flags = 0;

  sSigAction.sa_handler = SignalHandler;
  if(sigaction(SIGINT, &sSigAction, NULL) != 0)
  {
    fprintf(stderr, LIVEDRIVE_DECODER "unable to install handler for SIGINT\n");
    return -1;
  }
  return 0;
}


/*****************************************************************************
 * main function
 *****************************************************************************/
int main(int i_argc, char *ppsz_argv[], char *ppsz_env[])
{
  int i_rc = 0;
  main_thread_t main_thread;
  main_thread.b_stop = 0;

  fprintf(stdout, "Live! Drive IR Remote control decoder - (C) 2001 Arnaud de Bossoreille de Ribou\n\n");

  if((i_argc != 2) || !strcmp(ppsz_argv[1], "--help"))
  {
    fprintf(stdout,
      "Usage: sblive_decode device\n"
      "\n"
      "Arguments:\n"
      "  device: midi device where to read midi data (- == stdin)\n");
    return 0;;
  }

  /* Set static data pointer */
  s_p_main = &main_thread;

  /* Install signal handlers */
  if(!i_rc)
    i_rc = InstallSignalHandlers();

  /* Open the midi device */
  if(!i_rc)
  {
    if(strcmp(ppsz_argv[1], "-"))
    {
      main_thread.i_fd = open(ppsz_argv[1], O_RDONLY);
      i_rc = (main_thread.i_fd == -1) ? -1 : 0;
    }
    else
      main_thread.i_fd = 0; /* open stdin */

    if(i_rc)
      fprintf(stderr, LIVEDRIVE_DECODER "unable to open %s\n", ppsz_argv[1]);
  }

  if(!i_rc)
  {
    unsigned int ui;

    /* Initialize the main structure (lifo...) */
    for(ui = 0; ui < DD_LIFO_SIZE; ui++)
    {
      main_thread.p_packets[ui].p_buffer_start =
                                &main_thread.buffer[DD_PACKET_SIZE * ui];
      main_thread.p_packets[ui].p_buffer_end   =
                                &main_thread.buffer[DD_PACKET_SIZE * (ui + 1)];
      main_thread.lifo[ui] = &main_thread.p_packets[ui];
    }
    main_thread.ui_next = 0;

    /* Initialize the midi demultiplexer */
    douze_InitDemux(&main_thread.demux,
                    (douze_GetPacketFunc)GetPacket,
                    (douze_ReleasePacketFunc)ReleasePacket,
                    (douze_ParseMessageFunc)douze_ParseMessage,
                    &main_thread, &main_thread.parser);

    /* Initialize the midi parser */
    douze_InitParser(&main_thread.parser,
                     NULL,
                     NULL,
                     NULL,
                     NULL,
                     NULL,
                     NULL,
                     NULL,
                     (douze_OnSystemExclusiveFunc)livedrive_OnSystemExclusive,
                     NULL,
                     NULL,
                     NULL,
                     NULL,
                     NULL,
                     NULL,
                     NULL,
                     NULL,
                     NULL,
                     NULL,
                     NULL,
                     NULL,
                     &main_thread.decoder);

    /* Initialize the live drive decoder */
    livedrive_InitDecoder(&main_thread.decoder,
                          (livedrive_HandleKeyFunc)HandleKey,
                          NULL);

    /* main loop */
    while(!main_thread.b_stop)
    {
      byte_t data;
      if(read(main_thread.i_fd, &data, 1) == 1)
        douze_DemuxByte(&main_thread.demux, data);
      else
        main_thread.b_stop = 1;
    }
  }

  /* Close the midi device */
  if(main_thread.i_fd >= 0)
    i_rc |= close(main_thread.i_fd);

  /* Release the demultiplexer */
  douze_ReleaseDemux(&main_thread.demux);

  fprintf(stdout, LIVEDRIVE_DECODER "exiting ...\n");

  return i_rc;
}

